#ifndef INCLUDE_IP_H
#define INCLUDE_IP_H

/* system standard headers */
#include <sys/types.h>
#include <netinet/in.h>

#if defined(SUPPORT_CODE_ONLY)

#include <kern/if.h>
#include <kern/pbuf.h>
#include <kern/types.h>
#include <kern/ksocket.h>

#else

/* project specific headers */
#include <project3/include/if.h>
#include <project3/include/pbuf.h>
#include <project3/include/types.h>
#include <project3/include/ksocket.h>

/* your own headers */

#endif

#define IP_DEFAULT_TTL 64
#define IP_NOROUTE 2

/* variable declarations */
extern struct ifnet *default_if;
extern struct sockaddr_in in_ifaddr;

/* "flags" arg for ip_output() */
#define IP_FORWARDING           0x1

void ip_init();

/* Called by the transport layer or ICMP to send a packet.  Sets the
 * appropriate fields in the IP header.  Looks up the route table to
 * choose correct interface to send/forward the packet.  
 *
 * Returns SUCCESS if success, otherwise returns the appropriate error number.
 */
int ip_output(struct pbuf *pkt, int flags, u_char proto, 
	      struct in_addr saddr,
              struct in_addr daddr);

/* Called by the link layer when a packet is received.  Drop the
 * packet if it is ill formatted.  Generate an ICMP error
 * if transport layer (UDP) fails to deliver the packet
 * (i.e. no process listening on the destination port.)
 * If the packet is destined for this host, deliver it to
 * the appropriate transport layer based on the protocol field.  If
 * the packet is not destined for this host, call ip_forward() to
 * forward the packet based on the destination address .
 */

void ip_input(struct ifnet *ifp, struct pbuf *pkt);

/* Decrement TTL.  Generate an ICMP error if TTL expires.
 * Forward the packet if everything is fine.
 */
void ip_forward(struct pbuf *pkt);


#endif
